@@ -1,4 +1,4 @@
-/* Copyright (C) 1995,1996,1997,1998,1999,2000,2001, 2002, 2003, 2004 Free Software Foundation, Inc.
+/* Copyright (C) 1995,1996,1997,1998,1999,2000,2001, 2002, 2003, 2004, 2006, 2008 Free Software Foundation, Inc.
  * 
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -12,11 +12,13 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 
 
+#define _LARGEFILE64_SOURCE      /* ask for stat64 etc */
+
 #if HAVE_CONFIG_H
 #  include <config.h>
 #endif
@@ -37,8 +39,6 @@
 #endif
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
-#else
-size_t fwrite ();
 #endif
 #ifdef HAVE_IO_H
 #include <io.h>
@@ -48,6 +48,7 @@ size_t fwrite ();
 #endif
 
 #include <errno.h>
+#include <sys/types.h>
 
 #include "libguile/iselect.h"
 
@@ -55,9 +56,33 @@ size_t fwrite ();
 #ifdef __MINGW32__
 # include <sys/stat.h>
 # include <winsock2.h>
-# define ftruncate(fd, size) chsize (fd, size)
 #endif /* __MINGW32__ */
 
+/* Mingw (version 3.4.5, circa 2006) has ftruncate as an alias for chsize
+   already, but have this code here in case that wasn't so in past versions,
+   or perhaps to help other minimal DOS environments.
+
+   gnulib ftruncate.c has code using fcntl F_CHSIZE and F_FREESP, which
+   might be possibilities if we've got other systems without ftruncate.  */
+
+#if HAVE_CHSIZE && ! HAVE_FTRUNCATE
+# define ftruncate(fd, size) chsize (fd, size)
+#undef HAVE_FTRUNCATE
+#define HAVE_FTRUNCATE 1
+#endif
+
+#if SIZEOF_OFF_T == SIZEOF_INT
+#define OFF_T_MAX  INT_MAX
+#define OFF_T_MIN  INT_MIN
+#elif SIZEOF_OFF_T == SIZEOF_LONG
+#define OFF_T_MAX  LONG_MAX
+#define OFF_T_MIN  LONG_MIN
+#elif SIZEOF_OFF_T == SIZEOF_LONG_LONG
+#define OFF_T_MAX  LONG_LONG_MAX
+#define OFF_T_MIN  LONG_LONG_MIN
+#else
+#error Oops, unknown OFF_T size
+#endif
 
 scm_t_bits scm_tc16_fport;
 
@@ -155,7 +180,7 @@ SCM_DEFINE (scm_setvbuf, "setvbuf", 2, 1, 0,
     }
   else
     {
-      SCM_SET_CELL_WORD_0 (port, SCM_CELL_WORD_0 (port) ^ SCM_BUFLINE);
+      SCM_SET_CELL_WORD_0 (port, SCM_CELL_WORD_0 (port) & ~(scm_t_bits)SCM_BUFLINE);
     }
 
   if (SCM_UNBNDP (size))
@@ -264,6 +289,8 @@ SCM_DEFINE (scm_open_file, "open-file", 2, 0, 0,
 	    "@end table\n"
 	    "The following additional characters can be appended:\n"
 	    "@table @samp\n"
+	    "@item b\n"
+	    "Open the underlying file in binary mode, if supported by the operating system. "
 	    "@item +\n"
 	    "Open the port for both input and output.  E.g., @code{r+}: open\n"
 	    "an existing file for both input and output.\n"
@@ -291,13 +318,13 @@ SCM_DEFINE (scm_open_file, "open-file", 2, 0, 0,
   char *md;
   char *ptr;
 
-  scm_frame_begin (0);
+  scm_dynwind_begin (0);
 
   file = scm_to_locale_string (filename);
-  scm_frame_free (file);
+  scm_dynwind_free (file);
 
   md = scm_to_locale_string (mode);
-  scm_frame_free (md);
+  scm_dynwind_free (md);
 
   switch (*md)
     {
@@ -334,7 +361,7 @@ SCM_DEFINE (scm_open_file, "open-file", 2, 0, 0,
 	}
       ptr++;
     }
-  SCM_SYSCALL (fdes = open (file, flags, 0666));
+  SCM_SYSCALL (fdes = open_or_open64 (file, flags, 0666));
   if (fdes == -1)
     {
       int en = errno;
@@ -345,7 +372,7 @@ SCM_DEFINE (scm_open_file, "open-file", 2, 0, 0,
     }
   port = scm_i_fdes_to_port (fdes, scm_i_mode_bits (mode), filename);
 
-  scm_frame_end ();
+  scm_dynwind_end ();
 
   return port;
 }
@@ -458,9 +485,8 @@ scm_fdes_to_port (int fdes, char *mode, SCM name)
 static int
 fport_input_waiting (SCM port)
 {
-  int fdes = SCM_FSTREAM (port)->fdes;
-
 #ifdef HAVE_SELECT
+  int fdes = SCM_FSTREAM (port)->fdes;
   struct timeval timeout;
   SELECT_TYPE read_set;
   SELECT_TYPE write_set;
@@ -480,10 +506,15 @@ fport_input_waiting (SCM port)
       < 0)
     scm_syserror ("fport_input_waiting");
   return FD_ISSET (fdes, &read_set) ? 1 : 0;
-#elif defined (FIONREAD)
+
+#elif HAVE_IOCTL && defined (FIONREAD)
+  /* Note: cannot test just defined(FIONREAD) here, since mingw has FIONREAD
+     (for use with winsock ioctlsocket()) but not ioctl().  */
+  int fdes = SCM_FSTREAM (port)->fdes;
   int remir;
   ioctl(fdes, FIONREAD, &remir);
   return remir;
+
 #else    
   scm_misc_error ("fport_input_waiting",
 		  "Not fully implemented on this platform",
@@ -579,25 +610,25 @@ fport_fill_input (SCM port)
     }
 }
 
-static off_t
-fport_seek (SCM port, off_t offset, int whence)
+static off_t_or_off64_t
+fport_seek_or_seek64 (SCM port, off_t_or_off64_t offset, int whence)
 {
   scm_t_port *pt = SCM_PTAB_ENTRY (port);
   scm_t_fport *fp = SCM_FSTREAM (port);
-  off_t rv;
-  off_t result;
+  off_t_or_off64_t rv;
+  off_t_or_off64_t result;
 
   if (pt->rw_active == SCM_PORT_WRITE)
     {
       if (offset != 0 || whence != SEEK_CUR)
 	{
 	  fport_flush (port);
-	  result = rv = lseek (fp->fdes, offset, whence);
+	  result = rv = lseek_or_lseek64 (fp->fdes, offset, whence);
 	}
       else
 	{
 	  /* read current position without disturbing the buffer.  */
-	  rv = lseek (fp->fdes, offset, whence);
+	  rv = lseek_or_lseek64 (fp->fdes, offset, whence);
 	  result = rv + (pt->write_pos - pt->write_buf);
 	}
     }
@@ -607,13 +638,13 @@ fport_seek (SCM port, off_t offset, int whence)
 	{
 	  /* could expand to avoid a second seek.  */
 	  scm_end_input (port);
-	  result = rv = lseek (fp->fdes, offset, whence);
+	  result = rv = lseek_or_lseek64 (fp->fdes, offset, whence);
 	}
       else
 	{
 	  /* read current position without disturbing the buffer
 	     (particularly the unread-char buffer).  */
-	  rv = lseek (fp->fdes, offset, whence);
+	  rv = lseek_or_lseek64 (fp->fdes, offset, whence);
 	  result = rv - (pt->read_end - pt->read_pos);
 
 	  if (pt->read_buf == pt->putback_buf)
@@ -622,7 +653,7 @@ fport_seek (SCM port, off_t offset, int whence)
     }
   else /* SCM_PORT_NEITHER */
     {
-      result = rv = lseek (fp->fdes, offset, whence);      
+      result = rv = lseek_or_lseek64 (fp->fdes, offset, whence);
     }
 
   if (rv == -1)
@@ -631,6 +662,39 @@ fport_seek (SCM port, off_t offset, int whence)
   return result;
 }
 
+/* If we've got largefile and off_t isn't already off64_t then
+   fport_seek_or_seek64 needs a range checking wrapper to be fport_seek in
+   the port descriptor.
+
+   Otherwise if no largefile, or off_t is the same as off64_t (which is the
+   case on NetBSD apparently), then fport_seek_or_seek64 is right to be
+   fport_seek already.  */
+
+#if GUILE_USE_64_CALLS && HAVE_STAT64 && SIZEOF_OFF_T != SIZEOF_OFF64_T
+static off_t
+fport_seek (SCM port, off_t offset, int whence)
+{
+  off64_t rv = fport_seek_or_seek64 (port, (off64_t) offset, whence);
+  if (rv > OFF_T_MAX || rv < OFF_T_MIN)
+    {
+      errno = EOVERFLOW;
+      scm_syserror ("fport_seek");
+    }
+  return (off_t) rv;
+
+}
+#else
+#define fport_seek   fport_seek_or_seek64
+#endif
+
+/* `how' has been validated and is one of SEEK_SET, SEEK_CUR or SEEK_END */
+SCM
+scm_i_fport_seek (SCM port, SCM offset, int how)
+{
+  return scm_from_off_t_or_off64_t
+    (fport_seek_or_seek64 (port, scm_to_off_t_or_off64_t (offset), how));
+}
+
 static void
 fport_truncate (SCM port, off_t length)
 {
@@ -640,6 +704,13 @@ fport_truncate (SCM port, off_t length)
     scm_syserror ("ftruncate");
 }
 
+int
+scm_i_fport_truncate (SCM port, SCM length)
+{
+  scm_t_fport *fp = SCM_FSTREAM (port);
+  return ftruncate_or_ftruncate64 (fp->fdes, scm_to_off_t_or_off64_t (length));
+}
+
 /* helper for fport_write: try to write data, using multiple system
    calls if required.  */
 #define FUNC_NAME "write_all"
