@@ -1,4 +1,4 @@
-/* Copyright (C) 1995,1996,1998,1999,2000,2001 Free Software Foundation, Inc.
+/* Copyright (C) 1995,1996,1998,1999,2000,2001, 2006 Free Software Foundation, Inc.
  * 
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -12,7 +12,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 
@@ -392,15 +392,15 @@ SCM_DEFINE (scm_vector_to_list, "vector->list", 1, 0, 0,
   SCM res = SCM_EOL;
   const SCM *data;
   scm_t_array_handle handle;
-  size_t i, len;
+  size_t i, count, len;
   ssize_t inc;
 
   data = scm_vector_elements (v, &handle, &len, &inc);
-  for (i = len*inc; i > 0;)
-    {
-      i -= inc;
-      res = scm_cons (data[i], res);
-    }
+  for (i = (len - 1) * inc, count = 0;
+       count < len;
+       i -= inc, count++)
+    res = scm_cons (data[i], res);
+
   scm_array_handle_release (&handle);
   return res;
 }
